@@ -1,6 +1,6 @@
 package List::Compare::Base::_Auxiliary;
-$VERSION = 0.33;
-# As of:  09/18/2005
+#$Id: _Auxiliary.pm 1329 2008-06-07 23:49:51Z jimk $
+$VERSION = 0.37;
 use Carp;
 @ISA = qw(Exporter);
 @EXPORT_OK = qw|
@@ -74,6 +74,8 @@ use Carp;
 use strict;
 local $^W =1;
 
+my $bad_lists_msg = q{If argument is single hash ref, you must have a 'lists' key whose value is an array ref};
+
 sub _validate_2_seenhashes {
     my ($refL, $refR) = @_;
     my (%seenL, %seenR);
@@ -92,20 +94,22 @@ sub _validate_2_seenhashes {
             $badentriesR{$_} = ${$refR}{$_};
         }
     }
+    my $msg = q{};
     if ( (keys %badentriesL) or (keys %badentriesR) ) {
-        print "\nValues in a 'seen-hash' may only be positive integers.\n";
-        print "  These elements have invalid values:\n\n";
+        $msg .= "\nValues in a 'seen-hash' may only be positive integers.\n";
+        $msg .= "  These elements have invalid values:\n";
         if (keys %badentriesL) {
-            print "  First hash in arguments:\n\n";
-            print "     Key:  $_\tValue:  $badentriesL{$_}\n" 
+            $msg .= "  First hash in arguments:\n";
+            $msg .= "     Key:  $_\tValue:  $badentriesL{$_}\n" 
                 foreach (sort keys %badentriesL);
         } 
         if (keys %badentriesR) {
-            print "  Second hash in arguments:\n\n";
-            print "     Key:  $_\tValue:  $badentriesR{$_}\n" 
+            $msg .= "  Second hash in arguments:\n";
+            $msg .= "     Key:  $_\tValue:  $badentriesR{$_}\n" 
                 foreach (sort keys %badentriesR);
         }
-        croak "Correct invalid values before proceeding:  $!";
+        $msg .= "Correct invalid values before proceeding";
+        croak "$msg:  $!";
     }
     return (\%seenL, \%seenR);
 }
@@ -124,20 +128,22 @@ sub _validate_seen_hash {
             $badentriesR{$_} = ${$r}{$_} 
                 unless (${$r}{$_} =~ /^\d+$/ and ${$r}{$_} > 0);
         }
+        my $msg = q{};
         if ( (keys %badentriesL) or (keys %badentriesR) ) {
-            print "\nValues in a 'seen-hash' may only be numeric.\n";
-            print "  These elements have invalid values:\n\n";
+            $msg .= "\nValues in a 'seen-hash' must be numeric.\n";
+            $msg .= "  These elements have invalid values:\n";
             if (keys %badentriesL) {
-                print "  First hash in arguments:\n\n";
-                print "     Key:  $_\tValue:  $badentriesL{$_}\n" 
+                $msg .= "  First hash in arguments:\n";
+                $msg .= "     Key:  $_\tValue:  $badentriesL{$_}\n" 
                     foreach (sort keys %badentriesL);
             } 
             if (keys %badentriesR) {
-                print "  Second hash in arguments:\n\n";
-                print "     Key:  $_\tValue:  $badentriesR{$_}\n" 
+                $msg .= "  Second hash in arguments:\n";
+                $msg .= "     Key:  $_\tValue:  $badentriesR{$_}\n" 
                     foreach (sort keys %badentriesR);
             }
-            croak "Correct invalid values before proceeding:  $!";
+            $msg .= "Correct invalid values before proceeding";
+            croak "$msg:  $!";
         }
     }
 }
@@ -155,17 +161,19 @@ sub _validate_multiple_seenhashes {
             }
         }
     }
+    my $msg = q{};
     if ($badentriesflag) {
-        print "\nValues in a 'seen-hash' may only be positive integers.\n";
-        print "  These elements have invalid values:\n\n";
+        $msg .= "\nValues in a 'seen-hash' must be positive integers.\n";
+        $msg .= "  These elements have invalid values:\n\n";
         foreach (sort keys %badentries) {
-            print "    Hash $_:\n";
+            $msg .= "    Hash $_:\n";
             my %pairs = %{$badentries{$_}};
             foreach my $val (sort keys %pairs) {
-                print "        Bad key-value pair:  $val\t$pairs{$val}\n";
+                $msg .= "        Bad key-value pair:  $val\t$pairs{$val}\n";
             }
         }
-        croak "Correct invalid values before proceeding:  $!";
+        $msg .= "Correct invalid values before proceeding";
+        croak "$msg:  $!";
     }
 }
 
@@ -388,7 +396,6 @@ sub _index_message1 {
     croak "Argument to method $method must be the array index of the target list \n  in list of arrays passed as arguments to the constructor: $!"
         unless (
                 $index =~ /^\d+$/ 
-           and  0 <= $index 
            and  $index <= ${$dataref}{'maxindex'}
         );
 }
@@ -408,7 +415,6 @@ sub _index_message2 {
             croak "Each argument to method $method must be a valid array index for the target list \n  in list of arrays passed as arguments to the constructor: $!"
                 unless (
                         $_ =~ /^\d+$/ 
-                   and  0 <= $_ 
                    and  $_ <= ${$dataref}{'maxindex'}
                 );
         }
@@ -422,7 +428,6 @@ sub _index_message3 {
     croak "Argument to method $method must be the array index of the target list \n  in list of arrays passed as arguments to the constructor: $!"
         unless (
                 $index =~ /^\d+$/ 
-           and  0 <= $index 
            and  $index <= $maxindex
         );
 }
@@ -442,7 +447,6 @@ sub _index_message4 {
             croak "Each argument to method $method must be a valid array index for the target list \n  in list of arrays passed as arguments to the constructor: $!"
                 unless (
                         $_ =~ /^\d+$/ 
-                   and  0 <= $_ 
                    and  $_ <= $maxindex
                 );
         }
@@ -471,15 +475,15 @@ sub _subset_engine_multaccel {
 
 sub _calc_seen {
     my ($refL, $refR) = @_;
-    if (ref($refL) eq 'ARRAY' and ref($refR) eq 'ARRAY') {
+    # We've already guaranteed that args are both array refs or both hash
+    # refs.  So checking the left-hand one is sufficient.
+    if (ref($refL) eq 'ARRAY') {
         my (%seenL, %seenR);
         foreach (@$refL) { $seenL{$_}++ }
         foreach (@$refR) { $seenR{$_}++ }
         return (\%seenL, \%seenR); 
-    } elsif (ref($refL) eq 'HASH' and ref($refR) eq 'HASH') {
-        return ($refL, $refR);
     } else {
-        croak "Improper mixing of arguments; accelerated calculation not possible:  $!";
+        return ($refL, $refR);
     }
 }
 
@@ -504,8 +508,6 @@ sub _equiv_engine {
 sub _argument_checker_0 {
     my @args = @_;
     my $first_ref = ref($args[0]);
-    die "Improper argument: $!" 
-        unless ($first_ref eq 'ARRAY' or $first_ref eq 'HASH');
     my @temp = @args[1..$#args];
     my ($testing);
     my $condition = 1;
@@ -523,6 +525,7 @@ sub _argument_checker_0 {
 
 sub _argument_checker {
     my $argref = shift;
+    croak "'$argref' must be an array ref" unless ref($argref) eq 'ARRAY';
     my @args = _argument_checker_0(@{$argref});
     return (@args);
 }
@@ -549,6 +552,8 @@ sub _argument_checker_2 {
 # stripped off).
 # The second argument is an arrayref holding a single item (index number of 
 # item being tested)
+# Note:  Currently we're only checking for the quantity of arguments -- not
+# their types.  This should be fixed.
 sub _argument_checker_3 {
     my $argref = shift;
     my @args = @{$argref};
@@ -557,8 +562,7 @@ sub _argument_checker_3 {
     } elsif (@args == 2) {
         return (_argument_checker($args[0]), ${$args[1]}[0]);
     } else {
-        croak "Subroutine call requires 1 or 2 references as arguments:  $!"
-            unless (@args == 1 or @args == 2);
+        croak "Subroutine call requires 1 or 2 references as arguments:  $!";
     }
 }
 
@@ -581,7 +585,7 @@ sub _argument_checker_4 {
         if (@{$args[1]} == 2) {
             my $last_index = $#{$args[0]};
             foreach my $i (@{$args[1]}) {
-		croak "No element in index position $i in list of list references passed as first argument to function: $!"
+		        croak "No element in index position $i in list of list references passed as first argument to function: $!"
                     unless ($i =~ /^\d+$/ and $i <= $last_index);
             }
             return (_argument_checker($args[0]), $args[1]);
@@ -589,8 +593,7 @@ sub _argument_checker_4 {
             croak "Must provide index positions corresponding to two lists: $!";
         }
     } else {
-        croak "Subroutine call requires 1 or 2 references as arguments: $!"
-            unless (@args == 1 or @args == 2);
+        croak "Subroutine call requires 1 or 2 references as arguments: $!";
     }
 }
 
@@ -611,10 +614,8 @@ sub _calc_seen1 {
             push(@seenrefs, \%seenthis);
         }
         return \@seenrefs;
-    } elsif (ref($listrefs[0]) eq 'HASH') {
-        return \@listrefs;
     } else {
-        croak "Indeterminate case in _calc_seen1: $!";
+        return \@listrefs;
     }
 }
 
@@ -625,7 +626,7 @@ sub _alt_construct_tester {
     my ($argref, $unsorted);
     if (@args == 1 and (ref($args[0]) eq 'HASH')) {
        my $hashref = shift;
-       die "Need to define 'lists' key properly: $!"
+       croak "$bad_lists_msg: $!"
            unless ( ${$hashref}{'lists'}
                 and (ref(${$hashref}{'lists'}) eq 'ARRAY') );
        $argref = ${$hashref}{'lists'};
@@ -646,10 +647,10 @@ sub _alt_construct_tester_1 {
     if (@args == 1 and (ref($args[0]) eq 'HASH')) {
         my (@returns);
         my $hashref = $args[0];
-        die "Need to define 'lists' key properly: $!"
+       croak "$bad_lists_msg: $!"
            unless ( ${$hashref}{'lists'}
                 and (ref(${$hashref}{'lists'}) eq 'ARRAY') );
-        die "Need to define 'item' key properly: $!"
+        croak "If argument is single hash ref, you must have an 'item' key: $!"
            unless ${$hashref}{'item'};
         @returns = ( ${$hashref}{'lists'}, [${$hashref}{'item'}] );
         $argref = \@returns;
@@ -663,24 +664,18 @@ sub _alt_construct_tester_1 {
 # are_members_which are_members_any
 sub _alt_construct_tester_2 {
     my @args = @_;
-    my ($argref);
     if (@args == 1 and (ref($args[0]) eq 'HASH')) {
-        my (@returns);
         my $hashref = $args[0];
-        die "Need to define 'lists' key properly: $!"
+       croak "$bad_lists_msg: $!"
            unless ( ${$hashref}{'lists'}
                 and (ref(${$hashref}{'lists'}) eq 'ARRAY') );
-        die "Need to define 'items' key properly: $!"
+        croak "If argument is single hash ref, you must have an 'items' key whose value is an array ref: $!"
            unless ( ${$hashref}{'items'}
                 and (ref(${$hashref}{'items'}) eq 'ARRAY') );
-        @returns = defined ${$hashref}{'items'}
-                        ? (${$hashref}{'lists'}, ${$hashref}{'items'})
-                        : (${$hashref}{'lists'});
-        $argref = \@returns;
+        return [ (${$hashref}{'lists'}, ${$hashref}{'items'}) ];
     } else {
-        $argref = \@args; 
+        return \@args;
     }
-    return $argref;
 }
 
 # _alt_construct_tester_3 prepares for _argument_checker_3 in
@@ -691,7 +686,7 @@ sub _alt_construct_tester_3 {
     if (@args == 1 and (ref($args[0]) eq 'HASH')) {
         my (@returns);
         my $hashref = $args[0];
-        die "Need to define 'lists' key properly: $!"
+       croak "$bad_lists_msg: $!"
            unless ( ${$hashref}{'lists'}
                 and (ref(${$hashref}{'lists'}) eq 'ARRAY') );
         @returns = defined ${$hashref}{'item'}
@@ -714,7 +709,7 @@ sub _alt_construct_tester_4 {
     if (@args == 1 and (ref($args[0]) eq 'HASH')) {
         my (@returns);
         my $hashref = $args[0];
-        die "Need to define 'lists' key properly: $!"
+       croak "$bad_lists_msg: $!"
            unless ( ${$hashref}{'lists'}
                 and (ref(${$hashref}{'lists'}) eq 'ARRAY') );
         @returns = defined ${$hashref}{'pair'}
@@ -735,7 +730,7 @@ sub _alt_construct_tester_5 {
     if (@args == 1) {
         if (ref($args[0]) eq 'HASH') {
            my $hashref = shift;
-           die "Need to define 'lists' key properly: $!"
+           croak "Need to define 'lists' key properly: $!"
                unless ( ${$hashref}{'lists'}
                     and (ref(${$hashref}{'lists'}) eq 'ARRAY') );
            $argref = ${$hashref}{'lists'};
@@ -758,8 +753,8 @@ List::Compare::Base::_Auxiliary - Internal use only
 
 =head1 VERSION
 
-This document refers to version 0.33 of List::Compare::Base::_Auxiliary.
-This version was released February 19, 2006.
+This document refers to version 0.37 of List::Compare::Base::_Auxiliary.
+This version was released June 07, 2008.
 
 =head1 SYNOPSIS
 
@@ -771,8 +766,8 @@ List::Compare::Functional.  They are not intended to be publicly callable.
 James E. Keenan (jkeenan@cpan.org).  When sending correspondence, please 
 include 'List::Compare' or 'List-Compare' in your subject line.
 
-Creation date:  May 20, 2002.  Last modification date:  February 19, 2006. 
-Copyright (c) 2002-04 James E. Keenan.  United States.  All rights reserved. 
+Creation date:  May 20, 2002.  Last modification date:  June 07, 2008. 
+Copyright (c) 2002-08 James E. Keenan.  United States.  All rights reserved. 
 This is free software and may be distributed under the same terms as Perl
 itself.
 
