@@ -1,5 +1,6 @@
 package List::Compare::Functional;
-$VERSION = 0.34;   # November 5, 2007 
+#$Id: Functional.pm 1329 2008-06-07 23:49:51Z jimk $
+$VERSION = 0.37;
 @ISA = qw(Exporter);
 @EXPORT_OK = qw|
     get_intersection
@@ -389,7 +390,7 @@ sub print_equivalence_chart {
 sub _print_equivalence_chart_engine {
     my $seenrefsref = _calc_seen1(@_);
     my $xequivalentref = _equivalent_subengine($seenrefsref);
-    my $title = 'Subset';
+    my $title = 'Equivalence';
     _chart_engine_multiple($xequivalentref, $title);
 }    
 
@@ -494,7 +495,7 @@ sub _bag_engine {
                 push(@bag, $el);
             }
         }
-    } elsif (ref($listrefs[0]) eq 'HASH') {
+    } else {
         foreach my $lref (@listrefs) {
             foreach my $key (keys %{$lref}) {
                 for (my $j=1; $j <= ${$lref}{$key}; $j++) {
@@ -502,8 +503,6 @@ sub _bag_engine {
                 }
             }
         }
-    } else {
-        croak "Indeterminate case in _bag_engine: $!";
     }
     return \@bag;
 }
@@ -522,8 +521,8 @@ List::Compare::Functional - Compare elements of two or more lists
 
 =head1 VERSION
 
-This document refers to version 0.34 of List::Compare::Functional.  
-This version was released November 5, 2007.  The first released 
+This document refers to version 0.37 of List::Compare::Functional.  
+This version was released June 07, 2008.  The first released 
 version of List::Compare::Functional was v0.21.  Its version numbers 
 are set to be consistent with the other parts of the List::Compare 
 distribution.
@@ -1687,8 +1686,8 @@ James E. Keenan (jkeenan@cpan.org).  When sending correspondence, please
 include 'List::Compare::Functional' or 'List-Compare-Functional' in your 
 subject line.
 
-Creation date:  May 20, 2002.  Last modification date:  November 5, 2007. 
-Copyright (c) 2002-04 James E. Keenan.  United States.  All rights reserved. 
+Creation date:  May 20, 2002.  Last modification date:  June 07, 2008. 
+Copyright (c) 2002-08 James E. Keenan.  United States.  All rights reserved. 
 This is free software and may be distributed under the same terms as Perl
 itself.
 
